# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ErrorEntity
from ..models import NotificationProcessResult
from ..models import PaymentNotificationPagingSlicedResult
from ..models import PaymentOrderChargeRequest
from ..models import PaymentOrderChargeStatus
from ..models import PaymentOrderCreate
from ..models import PaymentOrderInfo
from ..models import PaymentOrderNotifySimulation
from ..models import PaymentOrderPagingSlicedResult
from ..models import PaymentOrderRefund
from ..models import ValidationErrorEntity

from ..operations.payment import ChargePaymentOrder
from ..operations.payment import CreateUserPaymentOrder
from ..operations.payment import GetPaymentOrder
from ..operations.payment import GetPaymentOrderChargeStatus
from ..operations.payment import ListExtOrderNoByExtTxId
from ..operations.payment import QueryPaymentNotifications
from ..operations.payment import (
    QueryPaymentNotificationsNotificationSourceEnum,
    QueryPaymentNotificationsStatusEnum,
)
from ..operations.payment import QueryPaymentOrders
from ..operations.payment import (
    QueryPaymentOrdersChannelEnum,
    QueryPaymentOrdersStatusEnum,
)
from ..operations.payment import RefundUserPaymentOrder
from ..operations.payment import SimulatePaymentOrderNotification
from ..models import NotificationProcessResultStatusEnum
from ..models import PaymentOrderChargeRequestPaymentProviderEnum
from ..models import PaymentOrderChargeStatusStatusEnum
from ..models import PaymentOrderCreateItemTypeEnum
from ..models import (
    PaymentOrderInfoChannelEnum,
    PaymentOrderInfoItemTypeEnum,
    PaymentOrderInfoPaymentProviderEnum,
    PaymentOrderInfoStatusEnum,
)
from ..models import (
    PaymentOrderNotifySimulationNotifyTypeEnum,
    PaymentOrderNotifySimulationPaymentProviderEnum,
)


@same_doc_as(ChargePaymentOrder)
def charge_payment_order(
    payment_order_no: str,
    body: Optional[PaymentOrderChargeRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Charge payment order without payment flow (chargePaymentOrder)

    [TEST FACILITY ONLY] Forbidden in live environment. Charge payment order without payment flow for unpaid payment order, usually for test usage to simulate real currency payment process.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=4 (UPDATE)
      *  Returns : payment order instance

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}

        method: PUT

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderChargeRequest in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderInfo (successful operation)

        400: Bad Request - ErrorEntity (33322: Payment provider [{paymentProvider}] not supported)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33171: Invalid payment order status [{status}] for payment order [{paymentOrderNo}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ChargePaymentOrder.create(
        payment_order_no=payment_order_no,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ChargePaymentOrder)
async def charge_payment_order_async(
    payment_order_no: str,
    body: Optional[PaymentOrderChargeRequest] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Charge payment order without payment flow (chargePaymentOrder)

    [TEST FACILITY ONLY] Forbidden in live environment. Charge payment order without payment flow for unpaid payment order, usually for test usage to simulate real currency payment process.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=4 (UPDATE)
      *  Returns : payment order instance

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}

        method: PUT

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderChargeRequest in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderInfo (successful operation)

        400: Bad Request - ErrorEntity (33322: Payment provider [{paymentProvider}] not supported)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33171: Invalid payment order status [{status}] for payment order [{paymentOrderNo}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ChargePaymentOrder.create(
        payment_order_no=payment_order_no,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateUserPaymentOrder)
def create_user_payment_order(
    user_id: str,
    body: Optional[PaymentOrderCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create payment order (createUserPaymentOrder)

    [SERVICE COMMUNICATION ONLY] This API is used to create payment order from justice service. The result contains the payment station url.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT", action=1 (CREATE)
      * It will be forbidden while the user is banned: PAYMENT_INITIATE or ORDER_AND_PAYMENT
      *  Returns : created order

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/payment/orders

        method: POST

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderCreate in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - PaymentOrderInfo (successful operation)

        400: Bad Request - ErrorEntity (33121: Recurring payment failed with code: [{errorCode}] and message: [{errorMessage}] by provider: [{provider}] | 33122: Subscription not match when create payment order)

        403: Forbidden - ErrorEntity (20016: action is banned)

        404: Not Found - ErrorEntity (36141: Currency [{currencyCode}] does not exist in namespace [{namespace}] | 33141: Payment Order [{paymentOrderNo}] does not exist | 33145: Recurring token not found | 20008: user [{userId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateUserPaymentOrder.create(
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateUserPaymentOrder)
async def create_user_payment_order_async(
    user_id: str,
    body: Optional[PaymentOrderCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create payment order (createUserPaymentOrder)

    [SERVICE COMMUNICATION ONLY] This API is used to create payment order from justice service. The result contains the payment station url.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT", action=1 (CREATE)
      * It will be forbidden while the user is banned: PAYMENT_INITIATE or ORDER_AND_PAYMENT
      *  Returns : created order

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT [CREATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/payment/orders

        method: POST

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderCreate in body

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        201: Created - PaymentOrderInfo (successful operation)

        400: Bad Request - ErrorEntity (33121: Recurring payment failed with code: [{errorCode}] and message: [{errorMessage}] by provider: [{provider}] | 33122: Subscription not match when create payment order)

        403: Forbidden - ErrorEntity (20016: action is banned)

        404: Not Found - ErrorEntity (36141: Currency [{currencyCode}] does not exist in namespace [{namespace}] | 33141: Payment Order [{paymentOrderNo}] does not exist | 33145: Recurring token not found | 20008: user [{userId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (20006: optimistic lock)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateUserPaymentOrder.create(
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPaymentOrder)
def get_payment_order(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment order (getPaymentOrder)

    Get payment order by paymentOrderNo.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : payment order instance

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderInfo (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentOrder.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPaymentOrder)
async def get_payment_order_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment order (getPaymentOrder)

    Get payment order by paymentOrderNo.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : payment order instance

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderInfo (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentOrder.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPaymentOrderChargeStatus)
def get_payment_order_charge_status(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment order charge status (getPaymentOrderChargeStatus)

    Get payment order charge status.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : payment order charge status

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}/status

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderChargeStatus (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentOrderChargeStatus.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPaymentOrderChargeStatus)
async def get_payment_order_charge_status_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get payment order charge status (getPaymentOrderChargeStatus)

    Get payment order charge status.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : payment order charge status

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}/status

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderChargeStatus (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentOrderChargeStatus.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListExtOrderNoByExtTxId)
def list_ext_order_no_by_ext_tx_id(
    ext_tx_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List external order No by external transaction id (listExtOrderNoByExtTxId)

    List external order No by external transaction id.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : payment orders

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/byExtTxId

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        ext_tx_id: (extTxId) REQUIRED str in query

    Responses:
        200: OK - List[str] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListExtOrderNoByExtTxId.create(
        ext_tx_id=ext_tx_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListExtOrderNoByExtTxId)
async def list_ext_order_no_by_ext_tx_id_async(
    ext_tx_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List external order No by external transaction id (listExtOrderNoByExtTxId)

    List external order No by external transaction id.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : payment orders

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/byExtTxId

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        ext_tx_id: (extTxId) REQUIRED str in query

    Responses:
        200: OK - List[str] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListExtOrderNoByExtTxId.create(
        ext_tx_id=ext_tx_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryPaymentNotifications)
def query_payment_notifications(
    end_date: Optional[str] = None,
    external_id: Optional[str] = None,
    limit: Optional[int] = None,
    notification_source: Optional[
        Union[str, QueryPaymentNotificationsNotificationSourceEnum]
    ] = None,
    notification_type: Optional[str] = None,
    offset: Optional[int] = None,
    payment_order_no: Optional[str] = None,
    start_date: Optional[str] = None,
    status: Optional[Union[str, QueryPaymentNotificationsStatusEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query payment notifications (queryPaymentNotifications)

    Query payment notifications.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT:NOTIFICATION", action=2 (READ)
      *  Returns : Payment notifications

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT:NOTIFICATION [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/notifications

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_date: (endDate) OPTIONAL str in query

        external_id: (externalId) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        notification_source: (notificationSource) OPTIONAL Union[str, NotificationSourceEnum] in query

        notification_type: (notificationType) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        payment_order_no: (paymentOrderNo) OPTIONAL str in query

        start_date: (startDate) OPTIONAL str in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

    Responses:
        200: OK - PaymentNotificationPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryPaymentNotifications.create(
        end_date=end_date,
        external_id=external_id,
        limit=limit,
        notification_source=notification_source,
        notification_type=notification_type,
        offset=offset,
        payment_order_no=payment_order_no,
        start_date=start_date,
        status=status,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryPaymentNotifications)
async def query_payment_notifications_async(
    end_date: Optional[str] = None,
    external_id: Optional[str] = None,
    limit: Optional[int] = None,
    notification_source: Optional[
        Union[str, QueryPaymentNotificationsNotificationSourceEnum]
    ] = None,
    notification_type: Optional[str] = None,
    offset: Optional[int] = None,
    payment_order_no: Optional[str] = None,
    start_date: Optional[str] = None,
    status: Optional[Union[str, QueryPaymentNotificationsStatusEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query payment notifications (queryPaymentNotifications)

    Query payment notifications.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT:NOTIFICATION", action=2 (READ)
      *  Returns : Payment notifications

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT:NOTIFICATION [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/notifications

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_date: (endDate) OPTIONAL str in query

        external_id: (externalId) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        notification_source: (notificationSource) OPTIONAL Union[str, NotificationSourceEnum] in query

        notification_type: (notificationType) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        payment_order_no: (paymentOrderNo) OPTIONAL str in query

        start_date: (startDate) OPTIONAL str in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

    Responses:
        200: OK - PaymentNotificationPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryPaymentNotifications.create(
        end_date=end_date,
        external_id=external_id,
        limit=limit,
        notification_source=notification_source,
        notification_type=notification_type,
        offset=offset,
        payment_order_no=payment_order_no,
        start_date=start_date,
        status=status,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(QueryPaymentOrders)
def query_payment_orders(
    channel: Optional[Union[str, QueryPaymentOrdersChannelEnum]] = None,
    ext_tx_id: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    status: Optional[Union[str, QueryPaymentOrdersStatusEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query payment orders (queryPaymentOrders)

    Query payment orders.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : query payment orders

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        channel: (channel) OPTIONAL Union[str, ChannelEnum] in query

        ext_tx_id: (extTxId) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

    Responses:
        200: OK - PaymentOrderPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryPaymentOrders.create(
        channel=channel,
        ext_tx_id=ext_tx_id,
        limit=limit,
        offset=offset,
        status=status,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(QueryPaymentOrders)
async def query_payment_orders_async(
    channel: Optional[Union[str, QueryPaymentOrdersChannelEnum]] = None,
    ext_tx_id: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    status: Optional[Union[str, QueryPaymentOrdersStatusEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query payment orders (queryPaymentOrders)

    Query payment orders.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=2 (READ)
      *  Returns : query payment orders

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders

        method: GET

        tags: ["Payment"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        channel: (channel) OPTIONAL Union[str, ChannelEnum] in query

        ext_tx_id: (extTxId) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

    Responses:
        200: OK - PaymentOrderPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = QueryPaymentOrders.create(
        channel=channel,
        ext_tx_id=ext_tx_id,
        limit=limit,
        offset=offset,
        status=status,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RefundUserPaymentOrder)
def refund_user_payment_order(
    payment_order_no: str,
    user_id: str,
    body: Optional[PaymentOrderRefund] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Refund payment order (refundUserPaymentOrder)

    [SERVICE COMMUNICATION ONLY] This API is used to refund order by paymentOrderNo from justice service.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT", action=4 (UPDATE)

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/payment/orders/{paymentOrderNo}/refund

        method: PUT

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderRefund in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderInfo (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33172: Payment order [{paymentOrderNo}] is not refundable)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RefundUserPaymentOrder.create(
        payment_order_no=payment_order_no,
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RefundUserPaymentOrder)
async def refund_user_payment_order_async(
    payment_order_no: str,
    user_id: str,
    body: Optional[PaymentOrderRefund] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Refund payment order (refundUserPaymentOrder)

    [SERVICE COMMUNICATION ONLY] This API is used to refund order by paymentOrderNo from justice service.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT", action=4 (UPDATE)

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:USER:{userId}:PAYMENT [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/users/{userId}/payment/orders/{paymentOrderNo}/refund

        method: PUT

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderRefund in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK - PaymentOrderInfo (successful operation)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)

        409: Conflict - ErrorEntity (33172: Payment order [{paymentOrderNo}] is not refundable)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RefundUserPaymentOrder.create(
        payment_order_no=payment_order_no,
        user_id=user_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(SimulatePaymentOrderNotification)
def simulate_payment_order_notification(
    payment_order_no: str,
    body: Optional[PaymentOrderNotifySimulation] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Simulate payment notification (simulatePaymentOrderNotification)

    [TEST FACILITY ONLY] Forbidden in live environment. Simulate payment notification on sandbox payment order, usually for test usage to simulate real currency payment notification.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=4 (UPDATE)
      *  Returns : notification process result

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}/simulate-notification

        method: PUT

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderNotifySimulation in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - NotificationProcessResult (successful operation)

        400: Bad Request - ErrorEntity (33322: Payment provider [{paymentProvider}] not supported)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SimulatePaymentOrderNotification.create(
        payment_order_no=payment_order_no,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(SimulatePaymentOrderNotification)
async def simulate_payment_order_notification_async(
    payment_order_no: str,
    body: Optional[PaymentOrderNotifySimulation] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Simulate payment notification (simulatePaymentOrderNotification)

    [TEST FACILITY ONLY] Forbidden in live environment. Simulate payment notification on sandbox payment order, usually for test usage to simulate real currency payment notification.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:PAYMENT", action=4 (UPDATE)
      *  Returns : notification process result

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:PAYMENT [UPDATE]

    Properties:
        url: /platform/admin/namespaces/{namespace}/payment/orders/{paymentOrderNo}/simulate-notification

        method: PUT

        tags: ["Payment"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL PaymentOrderNotifySimulation in body

        namespace: (namespace) REQUIRED str in path

        payment_order_no: (paymentOrderNo) REQUIRED str in path

    Responses:
        200: OK - NotificationProcessResult (successful operation)

        400: Bad Request - ErrorEntity (33322: Payment provider [{paymentProvider}] not supported)

        404: Not Found - ErrorEntity (33141: Payment Order [{paymentOrderNo}] does not exist)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SimulatePaymentOrderNotification.create(
        payment_order_no=payment_order_no,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "4.25.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# achievement_platform
from .wrappers import get_xbl_user_achievements
from .wrappers import get_xbl_user_achievements_async
from .wrappers import unlock_steam_user_achievement
from .wrappers import unlock_steam_user_achievement_async
from .wrappers import update_xbl_user_achievement
from .wrappers import update_xbl_user_achievement_async

# anonymization
from .wrappers import anonymize_campaign
from .wrappers import anonymize_campaign_async
from .wrappers import anonymize_entitlement
from .wrappers import anonymize_entitlement_async
from .wrappers import anonymize_fulfillment
from .wrappers import anonymize_fulfillment_async
from .wrappers import anonymize_integration
from .wrappers import anonymize_integration_async
from .wrappers import anonymize_order
from .wrappers import anonymize_order_async
from .wrappers import anonymize_payment
from .wrappers import anonymize_payment_async
from .wrappers import anonymize_revocation
from .wrappers import anonymize_revocation_async
from .wrappers import anonymize_subscription
from .wrappers import anonymize_subscription_async
from .wrappers import anonymize_wallet
from .wrappers import anonymize_wallet_async

# campaign
from .wrappers import apply_user_redemption
from .wrappers import apply_user_redemption_async
from .wrappers import bulk_disable_codes
from .wrappers import bulk_disable_codes_async
from .wrappers import bulk_enable_codes
from .wrappers import bulk_enable_codes_async
from .wrappers import create_campaign
from .wrappers import create_campaign_async
from .wrappers import create_codes
from .wrappers import create_codes_async
from .wrappers import disable_code
from .wrappers import disable_code_async
from .wrappers import download
from .wrappers import download_async
from .wrappers import enable_code
from .wrappers import enable_code_async
from .wrappers import get_campaign
from .wrappers import get_campaign_async
from .wrappers import get_campaign_dynamic
from .wrappers import get_campaign_dynamic_async
from .wrappers import get_code
from .wrappers import get_code_async
from .wrappers import query_campaigns
from .wrappers import query_campaigns_async
from .wrappers import query_codes
from .wrappers import query_codes_async
from .wrappers import query_redeem_history
from .wrappers import query_redeem_history_async
from .wrappers import update_campaign
from .wrappers import update_campaign_async

# catalog_changes
from .wrappers import get_statistic
from .wrappers import get_statistic_async
from .wrappers import publish_all
from .wrappers import publish_all_async
from .wrappers import publish_selected
from .wrappers import publish_selected_async
from .wrappers import query_changes
from .wrappers import query_changes_async
from .wrappers import select_all_records
from .wrappers import select_all_records_async
from .wrappers import select_record
from .wrappers import select_record_async
from .wrappers import unselect_all_records
from .wrappers import unselect_all_records_async
from .wrappers import unselect_record
from .wrappers import unselect_record_async

# category
from .wrappers import create_category
from .wrappers import create_category_async
from .wrappers import delete_category
from .wrappers import delete_category_async
from .wrappers import download_categories
from .wrappers import download_categories_async
from .wrappers import get_category
from .wrappers import get_category_async
from .wrappers import get_child_categories
from .wrappers import get_child_categories_async
from .wrappers import get_descendant_categories
from .wrappers import get_descendant_categories_async
from .wrappers import get_root_categories
from .wrappers import get_root_categories_async
from .wrappers import list_categories_basic
from .wrappers import list_categories_basic_async
from .wrappers import public_get_category
from .wrappers import public_get_category_async
from .wrappers import public_get_child_categories
from .wrappers import public_get_child_categories_async
from .wrappers import public_get_descendant_categories
from .wrappers import public_get_descendant_categories_async
from .wrappers import public_get_root_categories
from .wrappers import public_get_root_categories_async
from .wrappers import update_category
from .wrappers import update_category_async

# currency
from .wrappers import create_currency
from .wrappers import create_currency_async
from .wrappers import delete_currency
from .wrappers import delete_currency_async
from .wrappers import get_currency_config
from .wrappers import get_currency_config_async
from .wrappers import get_currency_summary
from .wrappers import get_currency_summary_async
from .wrappers import list_currencies
from .wrappers import list_currencies_async
from .wrappers import public_list_currencies
from .wrappers import public_list_currencies_async
from .wrappers import update_currency
from .wrappers import update_currency_async

# dlc
from .wrappers import delete_dlc_item_config
from .wrappers import delete_dlc_item_config_async
from .wrappers import delete_platform_dlc_config
from .wrappers import delete_platform_dlc_config_async
from .wrappers import get_dlc_item_config
from .wrappers import get_dlc_item_config_async
from .wrappers import get_platform_dlc_config
from .wrappers import get_platform_dlc_config_async
from .wrappers import get_user_dlc
from .wrappers import get_user_dlc_async
from .wrappers import public_sync_psn_dlc_inventory
from .wrappers import public_sync_psn_dlc_inventory_async
from .wrappers import public_sync_psn_dlc_inventory_with_multiple_service_labels
from .wrappers import public_sync_psn_dlc_inventory_with_multiple_service_labels_async
from .wrappers import sync_epic_game_dlc
from .wrappers import sync_epic_game_dlc_async
from .wrappers import sync_steam_dlc
from .wrappers import sync_steam_dlc_async
from .wrappers import sync_xbox_dlc
from .wrappers import sync_xbox_dlc_async
from .wrappers import update_dlc_item_config
from .wrappers import update_dlc_item_config_async
from .wrappers import update_platform_dlc_config
from .wrappers import update_platform_dlc_config_async

# entitlement
from .wrappers import consume_user_entitlement
from .wrappers import consume_user_entitlement_async
from .wrappers import disable_user_entitlement
from .wrappers import disable_user_entitlement_async
from .wrappers import enable_user_entitlement
from .wrappers import enable_user_entitlement_async
from .wrappers import exists_any_user_active_entitlement
from .wrappers import exists_any_user_active_entitlement_async
from .wrappers import exists_any_user_active_entitlement_by_item_ids
from .wrappers import exists_any_user_active_entitlement_by_item_ids_async
from .wrappers import get_entitlement
from .wrappers import get_entitlement_async
from .wrappers import get_user_active_entitlements_by_item_ids
from .wrappers import get_user_active_entitlements_by_item_ids_async
from .wrappers import get_user_app_entitlement_by_app_id
from .wrappers import get_user_app_entitlement_by_app_id_async
from .wrappers import get_user_app_entitlement_ownership_by_app_id
from .wrappers import get_user_app_entitlement_ownership_by_app_id_async
from .wrappers import get_user_entitlement
from .wrappers import get_user_entitlement_async
from .wrappers import get_user_entitlement_by_item_id
from .wrappers import get_user_entitlement_by_item_id_async
from .wrappers import get_user_entitlement_by_sku
from .wrappers import get_user_entitlement_by_sku_async
from .wrappers import get_user_entitlement_histories
from .wrappers import get_user_entitlement_histories_async
from .wrappers import get_user_entitlement_ownership_by_item_id
from .wrappers import get_user_entitlement_ownership_by_item_id_async
from .wrappers import get_user_entitlement_ownership_by_item_ids
from .wrappers import get_user_entitlement_ownership_by_item_ids_async
from .wrappers import get_user_entitlement_ownership_by_sku
from .wrappers import get_user_entitlement_ownership_by_sku_async
from .wrappers import grant_user_entitlement
from .wrappers import grant_user_entitlement_async
from .wrappers import public_consume_user_entitlement
from .wrappers import public_consume_user_entitlement_async
from .wrappers import public_exists_any_my_active_entitlement
from .wrappers import public_exists_any_my_active_entitlement_async
from .wrappers import public_exists_any_user_active_entitlement
from .wrappers import public_exists_any_user_active_entitlement_async
from .wrappers import public_get_entitlement_ownership_token
from .wrappers import public_get_entitlement_ownership_token_async
from .wrappers import public_get_my_app_entitlement_ownership_by_app_id
from .wrappers import public_get_my_app_entitlement_ownership_by_app_id_async
from .wrappers import public_get_my_entitlement_ownership_by_item_id
from .wrappers import public_get_my_entitlement_ownership_by_item_id_async
from .wrappers import public_get_my_entitlement_ownership_by_sku
from .wrappers import public_get_my_entitlement_ownership_by_sku_async
from .wrappers import public_get_user_app_entitlement_by_app_id
from .wrappers import public_get_user_app_entitlement_by_app_id_async
from .wrappers import public_get_user_app_entitlement_ownership_by_app_id
from .wrappers import public_get_user_app_entitlement_ownership_by_app_id_async
from .wrappers import public_get_user_entitlement
from .wrappers import public_get_user_entitlement_async
from .wrappers import public_get_user_entitlement_by_item_id
from .wrappers import public_get_user_entitlement_by_item_id_async
from .wrappers import public_get_user_entitlement_by_sku
from .wrappers import public_get_user_entitlement_by_sku_async
from .wrappers import public_get_user_entitlement_ownership_by_item_id
from .wrappers import public_get_user_entitlement_ownership_by_item_id_async
from .wrappers import public_get_user_entitlement_ownership_by_item_ids
from .wrappers import public_get_user_entitlement_ownership_by_item_ids_async
from .wrappers import public_get_user_entitlement_ownership_by_sku
from .wrappers import public_get_user_entitlement_ownership_by_sku_async
from .wrappers import public_query_user_entitlements
from .wrappers import public_query_user_entitlements_async
from .wrappers import public_query_user_entitlements_by_app_type
from .wrappers import public_query_user_entitlements_by_app_type_async
from .wrappers import query_entitlements
from .wrappers import query_entitlements_async
from .wrappers import query_user_entitlements
from .wrappers import query_user_entitlements_async
from .wrappers import query_user_entitlements_by_app_type
from .wrappers import query_user_entitlements_by_app_type_async
from .wrappers import revoke_user_entitlement
from .wrappers import revoke_user_entitlement_async
from .wrappers import revoke_user_entitlements
from .wrappers import revoke_user_entitlements_async
from .wrappers import update_user_entitlement
from .wrappers import update_user_entitlement_async

# fulfillment
from .wrappers import fulfill_item
from .wrappers import fulfill_item_async
from .wrappers import fulfill_rewards
from .wrappers import fulfill_rewards_async
from .wrappers import public_redeem_code
from .wrappers import public_redeem_code_async
from .wrappers import query_fulfillment_histories
from .wrappers import query_fulfillment_histories_async
from .wrappers import redeem_code
from .wrappers import redeem_code_async

# fulfillment_script
from .wrappers import create_fulfillment_script
from .wrappers import create_fulfillment_script_async
from .wrappers import delete_fulfillment_script
from .wrappers import delete_fulfillment_script_async
from .wrappers import get_fulfillment_script
from .wrappers import get_fulfillment_script_async
from .wrappers import list_fulfillment_scripts
from .wrappers import list_fulfillment_scripts_async
from .wrappers import test_fulfillment_script_eval
from .wrappers import test_fulfillment_script_eval_async
from .wrappers import update_fulfillment_script
from .wrappers import update_fulfillment_script_async

# iap
from .wrappers import delete_apple_iap_config
from .wrappers import delete_apple_iap_config_async
from .wrappers import delete_epic_games_iap_config
from .wrappers import delete_epic_games_iap_config_async
from .wrappers import delete_google_iap_config
from .wrappers import delete_google_iap_config_async
from .wrappers import delete_iap_item_config
from .wrappers import delete_iap_item_config_async
from .wrappers import delete_playstation_iap_config
from .wrappers import delete_playstation_iap_config_async
from .wrappers import delete_steam_iap_config
from .wrappers import delete_steam_iap_config_async
from .wrappers import delete_twitch_iap_config
from .wrappers import delete_twitch_iap_config_async
from .wrappers import delete_xbl_ap_config
from .wrappers import delete_xbl_ap_config_async
from .wrappers import get_apple_iap_config
from .wrappers import get_apple_iap_config_async
from .wrappers import get_epic_games_iap_config
from .wrappers import get_epic_games_iap_config_async
from .wrappers import get_google_iap_config
from .wrappers import get_google_iap_config_async
from .wrappers import get_iap_item_config
from .wrappers import get_iap_item_config_async
from .wrappers import get_iap_item_mapping
from .wrappers import get_iap_item_mapping_async
from .wrappers import get_play_station_iap_config
from .wrappers import get_play_station_iap_config_async
from .wrappers import get_steam_iap_config
from .wrappers import get_steam_iap_config_async
from .wrappers import get_twitch_iap_config
from .wrappers import get_twitch_iap_config_async
from .wrappers import get_xbl_iap_config
from .wrappers import get_xbl_iap_config_async
from .wrappers import mock_fulfill_iap_item
from .wrappers import mock_fulfill_iap_item_async
from .wrappers import public_fulfill_apple_iap_item
from .wrappers import public_fulfill_apple_iap_item_async
from .wrappers import public_fulfill_google_iap_item
from .wrappers import public_fulfill_google_iap_item_async
from .wrappers import public_reconcile_play_station_store
from .wrappers import public_reconcile_play_station_store_async
from .wrappers import public_reconcile_play_station_store_with_multiple_service_labels
from .wrappers import (
    public_reconcile_play_station_store_with_multiple_service_labels_async,
)
from .wrappers import query_all_user_iap_orders
from .wrappers import query_all_user_iap_orders_async
from .wrappers import query_user_iap_consume_history
from .wrappers import query_user_iap_consume_history_async
from .wrappers import query_user_iap_orders
from .wrappers import query_user_iap_orders_async
from .wrappers import sync_epic_games_inventory
from .wrappers import sync_epic_games_inventory_async
from .wrappers import sync_steam_inventory
from .wrappers import sync_steam_inventory_async
from .wrappers import sync_twitch_drops_entitlement
from .wrappers import sync_twitch_drops_entitlement_async
from .wrappers import sync_twitch_drops_entitlement_1
from .wrappers import sync_twitch_drops_entitlement_1_async
from .wrappers import sync_xbox_inventory
from .wrappers import sync_xbox_inventory_async
from .wrappers import update_apple_iap_config
from .wrappers import update_apple_iap_config_async
from .wrappers import update_epic_games_iap_config
from .wrappers import update_epic_games_iap_config_async
from .wrappers import update_google_iap_config
from .wrappers import update_google_iap_config_async
from .wrappers import update_google_p12_file
from .wrappers import update_google_p12_file_async
from .wrappers import update_iap_item_config
from .wrappers import update_iap_item_config_async
from .wrappers import update_playstation_iap_config
from .wrappers import update_playstation_iap_config_async
from .wrappers import update_steam_iap_config
from .wrappers import update_steam_iap_config_async
from .wrappers import update_twitch_iap_config
from .wrappers import update_twitch_iap_config_async
from .wrappers import update_xbl_bp_cert_file
from .wrappers import update_xbl_bp_cert_file_async
from .wrappers import update_xbl_iap_config
from .wrappers import update_xbl_iap_config_async

# invoice
from .wrappers import download_invoice_details
from .wrappers import download_invoice_details_async
from .wrappers import generate_invoice_summary
from .wrappers import generate_invoice_summary_async

# item
from .wrappers import acquire_item
from .wrappers import acquire_item_async
from .wrappers import bulk_get_locale_items
from .wrappers import bulk_get_locale_items_async
from .wrappers import create_item
from .wrappers import create_item_async
from .wrappers import create_item_type_config
from .wrappers import create_item_type_config_async
from .wrappers import defeature_item
from .wrappers import defeature_item_async
from .wrappers import delete_item
from .wrappers import delete_item_async
from .wrappers import delete_item_type_config
from .wrappers import delete_item_type_config_async
from .wrappers import disable_item
from .wrappers import disable_item_async
from .wrappers import enable_item
from .wrappers import enable_item_async
from .wrappers import feature_item
from .wrappers import feature_item_async
from .wrappers import get_app
from .wrappers import get_app_async
from .wrappers import get_available_predicate_types
from .wrappers import get_available_predicate_types_async
from .wrappers import get_bulk_item_id_by_skus
from .wrappers import get_bulk_item_id_by_skus_async
from .wrappers import get_item
from .wrappers import get_item_async
from .wrappers import get_item_by_app_id
from .wrappers import get_item_by_app_id_async
from .wrappers import get_item_by_sku
from .wrappers import get_item_by_sku_async
from .wrappers import get_item_dynamic_data
from .wrappers import get_item_dynamic_data_async
from .wrappers import get_item_id_by_sku
from .wrappers import get_item_id_by_sku_async
from .wrappers import get_item_type_config
from .wrappers import get_item_type_config_async
from .wrappers import get_locale_item
from .wrappers import get_locale_item_async
from .wrappers import get_locale_item_by_sku
from .wrappers import get_locale_item_by_sku_async
from .wrappers import list_basic_items_by_features
from .wrappers import list_basic_items_by_features_async
from .wrappers import list_item_type_configs
from .wrappers import list_item_type_configs_async
from .wrappers import public_bulk_get_items
from .wrappers import public_bulk_get_items_async
from .wrappers import public_get_app
from .wrappers import public_get_app_async
from .wrappers import public_get_item
from .wrappers import public_get_item_async
from .wrappers import public_get_item_by_app_id
from .wrappers import public_get_item_by_app_id_async
from .wrappers import public_get_item_by_sku
from .wrappers import public_get_item_by_sku_async
from .wrappers import public_get_item_dynamic_data
from .wrappers import public_get_item_dynamic_data_async
from .wrappers import public_query_items
from .wrappers import public_query_items_async
from .wrappers import public_search_items
from .wrappers import public_search_items_async
from .wrappers import public_validate_item_purchase_condition
from .wrappers import public_validate_item_purchase_condition_async
from .wrappers import query_items
from .wrappers import query_items_async
from .wrappers import query_items_1
from .wrappers import query_items_1_async
from .wrappers import query_uncategorized_items
from .wrappers import query_uncategorized_items_async
from .wrappers import return_item
from .wrappers import return_item_async
from .wrappers import search_item_type_config
from .wrappers import search_item_type_config_async
from .wrappers import search_items
from .wrappers import search_items_async
from .wrappers import sync_in_game_item
from .wrappers import sync_in_game_item_async
from .wrappers import update_app
from .wrappers import update_app_async
from .wrappers import update_item
from .wrappers import update_item_async
from .wrappers import update_item_purchase_condition
from .wrappers import update_item_purchase_condition_async
from .wrappers import update_item_type_config
from .wrappers import update_item_type_config_async
from .wrappers import validate_item_purchase_condition
from .wrappers import validate_item_purchase_condition_async

# key_group
from .wrappers import create_key_group
from .wrappers import create_key_group_async
from .wrappers import get_key_group
from .wrappers import get_key_group_async
from .wrappers import get_key_group_by_booth_name
from .wrappers import get_key_group_by_booth_name_async
from .wrappers import get_key_group_dynamic
from .wrappers import get_key_group_dynamic_async
from .wrappers import list_keys
from .wrappers import list_keys_async
from .wrappers import query_key_groups
from .wrappers import query_key_groups_async
from .wrappers import update_key_group
from .wrappers import update_key_group_async
from .wrappers import upload_keys
from .wrappers import upload_keys_async

# order
from .wrappers import admin_create_user_order
from .wrappers import admin_create_user_order_async
from .wrappers import count_of_purchased_item
from .wrappers import count_of_purchased_item_async
from .wrappers import download_user_order_receipt
from .wrappers import download_user_order_receipt_async
from .wrappers import fulfill_user_order
from .wrappers import fulfill_user_order_async
from .wrappers import get_order
from .wrappers import get_order_async
from .wrappers import get_order_statistics
from .wrappers import get_order_statistics_async
from .wrappers import get_user_order
from .wrappers import get_user_order_async
from .wrappers import get_user_order_grant
from .wrappers import get_user_order_grant_async
from .wrappers import get_user_order_histories
from .wrappers import get_user_order_histories_async
from .wrappers import process_user_order_notification
from .wrappers import process_user_order_notification_async
from .wrappers import public_cancel_user_order
from .wrappers import public_cancel_user_order_async
from .wrappers import public_create_user_order
from .wrappers import public_create_user_order_async
from .wrappers import public_download_user_order_receipt
from .wrappers import public_download_user_order_receipt_async
from .wrappers import public_get_user_order
from .wrappers import public_get_user_order_async
from .wrappers import public_get_user_order_histories
from .wrappers import public_get_user_order_histories_async
from .wrappers import public_query_user_orders
from .wrappers import public_query_user_orders_async
from .wrappers import query_orders
from .wrappers import query_orders_async
from .wrappers import query_user_orders
from .wrappers import query_user_orders_async
from .wrappers import refund_order
from .wrappers import refund_order_async
from .wrappers import update_user_order_status
from .wrappers import update_user_order_status_async

# order_dedicated
from .wrappers import sync_orders
from .wrappers import sync_orders_async

# payment
from .wrappers import charge_payment_order
from .wrappers import charge_payment_order_async
from .wrappers import create_user_payment_order
from .wrappers import create_user_payment_order_async
from .wrappers import get_payment_order
from .wrappers import get_payment_order_async
from .wrappers import get_payment_order_charge_status
from .wrappers import get_payment_order_charge_status_async
from .wrappers import list_ext_order_no_by_ext_tx_id
from .wrappers import list_ext_order_no_by_ext_tx_id_async
from .wrappers import query_payment_notifications
from .wrappers import query_payment_notifications_async
from .wrappers import query_payment_orders
from .wrappers import query_payment_orders_async
from .wrappers import refund_user_payment_order
from .wrappers import refund_user_payment_order_async
from .wrappers import simulate_payment_order_notification
from .wrappers import simulate_payment_order_notification_async

# payment_account
from .wrappers import public_delete_payment_account
from .wrappers import public_delete_payment_account_async
from .wrappers import public_get_payment_accounts
from .wrappers import public_get_payment_accounts_async

# payment_callback_config
from .wrappers import get_payment_callback_config
from .wrappers import get_payment_callback_config_async
from .wrappers import update_payment_callback_config
from .wrappers import update_payment_callback_config_async

# payment_config
from .wrappers import create_payment_provider_config
from .wrappers import create_payment_provider_config_async
from .wrappers import debug_matched_payment_merchant_config
from .wrappers import debug_matched_payment_merchant_config_async
from .wrappers import debug_matched_payment_provider_config
from .wrappers import debug_matched_payment_provider_config_async
from .wrappers import delete_payment_provider_config
from .wrappers import delete_payment_provider_config_async
from .wrappers import get_aggregate_payment_providers
from .wrappers import get_aggregate_payment_providers_async
from .wrappers import get_payment_merchant_config
from .wrappers import get_payment_merchant_config_async
from .wrappers import get_payment_tax_config
from .wrappers import get_payment_tax_config_async
from .wrappers import get_special_payment_providers
from .wrappers import get_special_payment_providers_async
from .wrappers import query_payment_provider_config
from .wrappers import query_payment_provider_config_async
from .wrappers import test_adyen_config
from .wrappers import test_adyen_config_async
from .wrappers import test_adyen_config_by_id
from .wrappers import test_adyen_config_by_id_async
from .wrappers import test_ali_pay_config
from .wrappers import test_ali_pay_config_async
from .wrappers import test_ali_pay_config_by_id
from .wrappers import test_ali_pay_config_by_id_async
from .wrappers import test_checkout_config
from .wrappers import test_checkout_config_async
from .wrappers import test_checkout_config_by_id
from .wrappers import test_checkout_config_by_id_async
from .wrappers import test_pay_pal_config
from .wrappers import test_pay_pal_config_async
from .wrappers import test_pay_pal_config_by_id
from .wrappers import test_pay_pal_config_by_id_async
from .wrappers import test_stripe_config
from .wrappers import test_stripe_config_async
from .wrappers import test_stripe_config_by_id
from .wrappers import test_stripe_config_by_id_async
from .wrappers import test_wx_pay_config
from .wrappers import test_wx_pay_config_async
from .wrappers import test_wx_pay_config_by_id
from .wrappers import test_wx_pay_config_by_id_async
from .wrappers import test_xsolla_config
from .wrappers import test_xsolla_config_async
from .wrappers import test_xsolla_config_by_id
from .wrappers import test_xsolla_config_by_id_async
from .wrappers import update_adyen_config
from .wrappers import update_adyen_config_async
from .wrappers import update_ali_pay_config
from .wrappers import update_ali_pay_config_async
from .wrappers import update_checkout_config
from .wrappers import update_checkout_config_async
from .wrappers import update_pay_pal_config
from .wrappers import update_pay_pal_config_async
from .wrappers import update_payment_provider_config
from .wrappers import update_payment_provider_config_async
from .wrappers import update_payment_tax_config
from .wrappers import update_payment_tax_config_async
from .wrappers import update_stripe_config
from .wrappers import update_stripe_config_async
from .wrappers import update_wx_pay_config
from .wrappers import update_wx_pay_config_async
from .wrappers import update_wx_pay_config_cert
from .wrappers import update_wx_pay_config_cert_async
from .wrappers import update_xsolla_config
from .wrappers import update_xsolla_config_async
from .wrappers import update_xsolla_ui_config
from .wrappers import update_xsolla_ui_config_async

# payment_dedicated
from .wrappers import create_payment_order_by_dedicated
from .wrappers import create_payment_order_by_dedicated_async
from .wrappers import refund_payment_order_by_dedicated
from .wrappers import refund_payment_order_by_dedicated_async
from .wrappers import sync_payment_orders
from .wrappers import sync_payment_orders_async

# payment_station
from .wrappers import get_payment_customization
from .wrappers import get_payment_customization_async
from .wrappers import get_payment_public_config
from .wrappers import get_payment_public_config_async
from .wrappers import get_payment_tax_value
from .wrappers import get_payment_tax_value_async
from .wrappers import pay
from .wrappers import pay_async
from .wrappers import public_check_payment_order_paid_status
from .wrappers import public_check_payment_order_paid_status_async
from .wrappers import public_get_payment_methods
from .wrappers import public_get_payment_methods_async
from .wrappers import public_get_payment_url
from .wrappers import public_get_payment_url_async
from .wrappers import public_get_qr_code
from .wrappers import public_get_qr_code_async
from .wrappers import public_get_unpaid_payment_order
from .wrappers import public_get_unpaid_payment_order_async
from .wrappers import public_normalize_payment_return_url
from .wrappers import public_normalize_payment_return_url_async

# revocation
from .wrappers import delete_revocation_config
from .wrappers import delete_revocation_config_async
from .wrappers import do_revocation
from .wrappers import do_revocation_async
from .wrappers import get_payment_callback_config_1
from .wrappers import get_payment_callback_config_1_async
from .wrappers import query_revocation_histories
from .wrappers import query_revocation_histories_async
from .wrappers import update_revocation_config
from .wrappers import update_revocation_config_async

# reward
from .wrappers import check_event_condition
from .wrappers import check_event_condition_async
from .wrappers import create_reward
from .wrappers import create_reward_async
from .wrappers import delete_reward
from .wrappers import delete_reward_async
from .wrappers import delete_reward_condition_record
from .wrappers import delete_reward_condition_record_async
from .wrappers import export_rewards
from .wrappers import export_rewards_async
from .wrappers import get_reward
from .wrappers import get_reward_async
from .wrappers import get_reward_1
from .wrappers import get_reward_1_async
from .wrappers import get_reward_by_code
from .wrappers import get_reward_by_code_async
from .wrappers import import_rewards
from .wrappers import import_rewards_async
from .wrappers import query_rewards
from .wrappers import query_rewards_async
from .wrappers import query_rewards_1
from .wrappers import query_rewards_1_async
from .wrappers import update_reward
from .wrappers import update_reward_async

# section
from .wrappers import create_section
from .wrappers import create_section_async
from .wrappers import delete_section
from .wrappers import delete_section_async
from .wrappers import get_section
from .wrappers import get_section_async
from .wrappers import public_list_active_sections
from .wrappers import public_list_active_sections_async
from .wrappers import purge_expired_section
from .wrappers import purge_expired_section_async
from .wrappers import query_sections
from .wrappers import query_sections_async
from .wrappers import update_section
from .wrappers import update_section_async

# service_plugin_config
from .wrappers import delete_service_plugin_config
from .wrappers import delete_service_plugin_config_async
from .wrappers import get_service_plugin_config
from .wrappers import get_service_plugin_config_async
from .wrappers import update_service_plugin_config
from .wrappers import update_service_plugin_config_async

# store
from .wrappers import clone_store
from .wrappers import clone_store_async
from .wrappers import create_store
from .wrappers import create_store_async
from .wrappers import delete_published_store
from .wrappers import delete_published_store_async
from .wrappers import delete_store
from .wrappers import delete_store_async
from .wrappers import export_store
from .wrappers import export_store_async
from .wrappers import export_store_1
from .wrappers import export_store_1_async
from .wrappers import get_published_store
from .wrappers import get_published_store_async
from .wrappers import get_published_store_backup
from .wrappers import get_published_store_backup_async
from .wrappers import get_store
from .wrappers import get_store_async
from .wrappers import import_store
from .wrappers import import_store_async
from .wrappers import import_store_1
from .wrappers import import_store_1_async
from .wrappers import list_stores
from .wrappers import list_stores_async
from .wrappers import public_list_stores
from .wrappers import public_list_stores_async
from .wrappers import rollback_published_store
from .wrappers import rollback_published_store_async
from .wrappers import update_store
from .wrappers import update_store_async

# subscription
from .wrappers import cancel_subscription
from .wrappers import cancel_subscription_async
from .wrappers import check_user_subscription_subscribable_by_item_id
from .wrappers import check_user_subscription_subscribable_by_item_id_async
from .wrappers import delete_user_subscription
from .wrappers import delete_user_subscription_async
from .wrappers import get_user_subscription
from .wrappers import get_user_subscription_async
from .wrappers import get_user_subscription_activities
from .wrappers import get_user_subscription_activities_async
from .wrappers import get_user_subscription_billing_histories
from .wrappers import get_user_subscription_billing_histories_async
from .wrappers import grant_days_to_subscription
from .wrappers import grant_days_to_subscription_async
from .wrappers import platform_subscribe_subscription
from .wrappers import platform_subscribe_subscription_async
from .wrappers import process_user_subscription_notification
from .wrappers import process_user_subscription_notification_async
from .wrappers import public_cancel_subscription
from .wrappers import public_cancel_subscription_async
from .wrappers import public_change_subscription_billing_account
from .wrappers import public_change_subscription_billing_account_async
from .wrappers import public_check_user_subscription_subscribable_by_item_id
from .wrappers import public_check_user_subscription_subscribable_by_item_id_async
from .wrappers import public_get_user_subscription
from .wrappers import public_get_user_subscription_async
from .wrappers import public_get_user_subscription_billing_histories
from .wrappers import public_get_user_subscription_billing_histories_async
from .wrappers import public_query_user_subscriptions
from .wrappers import public_query_user_subscriptions_async
from .wrappers import public_subscribe_subscription
from .wrappers import public_subscribe_subscription_async
from .wrappers import query_subscriptions
from .wrappers import query_subscriptions_async
from .wrappers import query_user_subscriptions
from .wrappers import query_user_subscriptions_async
from .wrappers import recurring_charge_subscription
from .wrappers import recurring_charge_subscription_async

# ticket
from .wrappers import acquire_user_ticket
from .wrappers import acquire_user_ticket_async
from .wrappers import decrease_ticket_sale
from .wrappers import decrease_ticket_sale_async
from .wrappers import get_ticket_booth_id
from .wrappers import get_ticket_booth_id_async
from .wrappers import get_ticket_dynamic
from .wrappers import get_ticket_dynamic_async
from .wrappers import increase_ticket_sale
from .wrappers import increase_ticket_sale_async

# view
from .wrappers import create_view
from .wrappers import create_view_async
from .wrappers import delete_view
from .wrappers import delete_view_async
from .wrappers import get_view
from .wrappers import get_view_async
from .wrappers import list_views
from .wrappers import list_views_async
from .wrappers import public_list_views
from .wrappers import public_list_views_async
from .wrappers import update_view
from .wrappers import update_view_async

# wallet
from .wrappers import check_wallet
from .wrappers import check_wallet_async
from .wrappers import credit_user_wallet
from .wrappers import credit_user_wallet_async
from .wrappers import debit_user_wallet
from .wrappers import debit_user_wallet_async
from .wrappers import debit_user_wallet_by_currency_code
from .wrappers import debit_user_wallet_by_currency_code_async
from .wrappers import disable_user_wallet
from .wrappers import disable_user_wallet_async
from .wrappers import enable_user_wallet
from .wrappers import enable_user_wallet_async
from .wrappers import get_platform_wallet_config
from .wrappers import get_platform_wallet_config_async
from .wrappers import get_user_wallet
from .wrappers import get_user_wallet_async
from .wrappers import get_wallet
from .wrappers import get_wallet_async
from .wrappers import list_user_currency_transactions
from .wrappers import list_user_currency_transactions_async
from .wrappers import list_user_wallet_transactions
from .wrappers import list_user_wallet_transactions_async
from .wrappers import pay_with_user_wallet
from .wrappers import pay_with_user_wallet_async
from .wrappers import public_get_my_wallet
from .wrappers import public_get_my_wallet_async
from .wrappers import public_get_wallet
from .wrappers import public_get_wallet_async
from .wrappers import public_list_user_wallet_transactions
from .wrappers import public_list_user_wallet_transactions_async
from .wrappers import query_user_currency_wallets
from .wrappers import query_user_currency_wallets_async
from .wrappers import query_wallets
from .wrappers import query_wallets_async
from .wrappers import reset_platform_wallet_config
from .wrappers import reset_platform_wallet_config_async
from .wrappers import update_platform_wallet_config
from .wrappers import update_platform_wallet_config_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service (4.25.0)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse
from .....core import StrEnum

from ...models import ErrorEntity
from ...models import FullItemPagingSlicedResult
from ...models import ValidationErrorEntity


class AppTypeEnum(StrEnum):
    DEMO = "DEMO"
    DLC = "DLC"
    GAME = "GAME"
    SOFTWARE = "SOFTWARE"


class ItemTypeEnum(StrEnum):
    APP = "APP"
    BUNDLE = "BUNDLE"
    CODE = "CODE"
    COINS = "COINS"
    EXTENSION = "EXTENSION"
    INGAMEITEM = "INGAMEITEM"
    LOOTBOX = "LOOTBOX"
    MEDIA = "MEDIA"
    OPTIONBOX = "OPTIONBOX"
    SEASON = "SEASON"
    SUBSCRIPTION = "SUBSCRIPTION"


class SortByEnum(StrEnum):
    NAME = "name"
    NAME_ASC = "name:asc"
    NAME_DESC = "name:desc"
    CREATEDAT = "createdAt"
    CREATEDAT_ASC = "createdAt:asc"
    CREATEDAT_DESC = "createdAt:desc"
    UPDATEDAT = "updatedAt"
    UPDATEDAT_ASC = "updatedAt:asc"
    UPDATEDAT_DESC = "updatedAt:desc"
    DISPLAYORDER = "displayOrder"
    DISPLAYORDER_ASC = "displayOrder:asc"
    DISPLAYORDER_DESC = "displayOrder:desc"


class QueryItems(Operation):
    """Query items by criteria (queryItems)

    This API is used to query items by criteria within a store.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:ITEM", action=2 (READ)
      *  Returns : the list of items

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:ITEM [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/items/byCriteria

        method: GET

        tags: ["Item"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        app_type: (appType) OPTIONAL Union[str, AppTypeEnum] in query

        available_date: (availableDate) OPTIONAL str in query

        base_app_id: (baseAppId) OPTIONAL str in query

        category_path: (categoryPath) OPTIONAL str in query

        features: (features) OPTIONAL str in query

        include_sub_category_item: (includeSubCategoryItem) OPTIONAL bool in query

        item_type: (itemType) OPTIONAL Union[str, ItemTypeEnum] in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        region: (region) OPTIONAL str in query

        sort_by: (sortBy) OPTIONAL List[Union[str, SortByEnum]] in query

        store_id: (storeId) OPTIONAL str in query

        tags: (tags) OPTIONAL str in query

        target_namespace: (targetNamespace) OPTIONAL str in query

    Responses:
        200: OK - FullItemPagingSlicedResult (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """

    # region fields

    _url: str = "/platform/admin/namespaces/{namespace}/items/byCriteria"
    _method: str = "GET"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    active_only: bool  # OPTIONAL in [query]
    app_type: Union[str, AppTypeEnum]  # OPTIONAL in [query]
    available_date: str  # OPTIONAL in [query]
    base_app_id: str  # OPTIONAL in [query]
    category_path: str  # OPTIONAL in [query]
    features: str  # OPTIONAL in [query]
    include_sub_category_item: bool  # OPTIONAL in [query]
    item_type: Union[str, ItemTypeEnum]  # OPTIONAL in [query]
    limit: int  # OPTIONAL in [query]
    offset: int  # OPTIONAL in [query]
    region: str  # OPTIONAL in [query]
    sort_by: List[Union[str, SortByEnum]]  # OPTIONAL in [query]
    store_id: str  # OPTIONAL in [query]
    tags: str  # OPTIONAL in [query]
    target_namespace: str  # OPTIONAL in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "active_only"):
            result["activeOnly"] = self.active_only
        if hasattr(self, "app_type"):
            result["appType"] = self.app_type
        if hasattr(self, "available_date"):
            result["availableDate"] = self.available_date
        if hasattr(self, "base_app_id"):
            result["baseAppId"] = self.base_app_id
        if hasattr(self, "category_path"):
            result["categoryPath"] = self.category_path
        if hasattr(self, "features"):
            result["features"] = self.features
        if hasattr(self, "include_sub_category_item"):
            result["includeSubCategoryItem"] = self.include_sub_category_item
        if hasattr(self, "item_type"):
            result["itemType"] = self.item_type
        if hasattr(self, "limit"):
            result["limit"] = self.limit
        if hasattr(self, "offset"):
            result["offset"] = self.offset
        if hasattr(self, "region"):
            result["region"] = self.region
        if hasattr(self, "sort_by"):
            result["sortBy"] = self.sort_by
        if hasattr(self, "store_id"):
            result["storeId"] = self.store_id
        if hasattr(self, "tags"):
            result["tags"] = self.tags
        if hasattr(self, "target_namespace"):
            result["targetNamespace"] = self.target_namespace
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> QueryItems:
        self.namespace = value
        return self

    def with_active_only(self, value: bool) -> QueryItems:
        self.active_only = value
        return self

    def with_app_type(self, value: Union[str, AppTypeEnum]) -> QueryItems:
        self.app_type = value
        return self

    def with_available_date(self, value: str) -> QueryItems:
        self.available_date = value
        return self

    def with_base_app_id(self, value: str) -> QueryItems:
        self.base_app_id = value
        return self

    def with_category_path(self, value: str) -> QueryItems:
        self.category_path = value
        return self

    def with_features(self, value: str) -> QueryItems:
        self.features = value
        return self

    def with_include_sub_category_item(self, value: bool) -> QueryItems:
        self.include_sub_category_item = value
        return self

    def with_item_type(self, value: Union[str, ItemTypeEnum]) -> QueryItems:
        self.item_type = value
        return self

    def with_limit(self, value: int) -> QueryItems:
        self.limit = value
        return self

    def with_offset(self, value: int) -> QueryItems:
        self.offset = value
        return self

    def with_region(self, value: str) -> QueryItems:
        self.region = value
        return self

    def with_sort_by(self, value: List[Union[str, SortByEnum]]) -> QueryItems:
        self.sort_by = value
        return self

    def with_store_id(self, value: str) -> QueryItems:
        self.store_id = value
        return self

    def with_tags(self, value: str) -> QueryItems:
        self.tags = value
        return self

    def with_target_namespace(self, value: str) -> QueryItems:
        self.target_namespace = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "active_only") and self.active_only:
            result["activeOnly"] = bool(self.active_only)
        elif include_empty:
            result["activeOnly"] = False
        if hasattr(self, "app_type") and self.app_type:
            result["appType"] = str(self.app_type)
        elif include_empty:
            result["appType"] = Union[str, AppTypeEnum]()
        if hasattr(self, "available_date") and self.available_date:
            result["availableDate"] = str(self.available_date)
        elif include_empty:
            result["availableDate"] = ""
        if hasattr(self, "base_app_id") and self.base_app_id:
            result["baseAppId"] = str(self.base_app_id)
        elif include_empty:
            result["baseAppId"] = ""
        if hasattr(self, "category_path") and self.category_path:
            result["categoryPath"] = str(self.category_path)
        elif include_empty:
            result["categoryPath"] = ""
        if hasattr(self, "features") and self.features:
            result["features"] = str(self.features)
        elif include_empty:
            result["features"] = ""
        if (
            hasattr(self, "include_sub_category_item")
            and self.include_sub_category_item
        ):
            result["includeSubCategoryItem"] = bool(self.include_sub_category_item)
        elif include_empty:
            result["includeSubCategoryItem"] = False
        if hasattr(self, "item_type") and self.item_type:
            result["itemType"] = str(self.item_type)
        elif include_empty:
            result["itemType"] = Union[str, ItemTypeEnum]()
        if hasattr(self, "limit") and self.limit:
            result["limit"] = int(self.limit)
        elif include_empty:
            result["limit"] = 0
        if hasattr(self, "offset") and self.offset:
            result["offset"] = int(self.offset)
        elif include_empty:
            result["offset"] = 0
        if hasattr(self, "region") and self.region:
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "sort_by") and self.sort_by:
            result["sortBy"] = [str(i0) for i0 in self.sort_by]
        elif include_empty:
            result["sortBy"] = []
        if hasattr(self, "store_id") and self.store_id:
            result["storeId"] = str(self.store_id)
        elif include_empty:
            result["storeId"] = ""
        if hasattr(self, "tags") and self.tags:
            result["tags"] = str(self.tags)
        elif include_empty:
            result["tags"] = ""
        if hasattr(self, "target_namespace") and self.target_namespace:
            result["targetNamespace"] = str(self.target_namespace)
        elif include_empty:
            result["targetNamespace"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, FullItemPagingSlicedResult],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        200: OK - FullItemPagingSlicedResult (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return FullItemPagingSlicedResult.create_from_dict(content), None
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)
        if code == 422:
            return None, ValidationErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        active_only: Optional[bool] = None,
        app_type: Optional[Union[str, AppTypeEnum]] = None,
        available_date: Optional[str] = None,
        base_app_id: Optional[str] = None,
        category_path: Optional[str] = None,
        features: Optional[str] = None,
        include_sub_category_item: Optional[bool] = None,
        item_type: Optional[Union[str, ItemTypeEnum]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        region: Optional[str] = None,
        sort_by: Optional[List[Union[str, SortByEnum]]] = None,
        store_id: Optional[str] = None,
        tags: Optional[str] = None,
        target_namespace: Optional[str] = None,
    ) -> QueryItems:
        instance = cls()
        instance.namespace = namespace
        if active_only is not None:
            instance.active_only = active_only
        if app_type is not None:
            instance.app_type = app_type
        if available_date is not None:
            instance.available_date = available_date
        if base_app_id is not None:
            instance.base_app_id = base_app_id
        if category_path is not None:
            instance.category_path = category_path
        if features is not None:
            instance.features = features
        if include_sub_category_item is not None:
            instance.include_sub_category_item = include_sub_category_item
        if item_type is not None:
            instance.item_type = item_type
        if limit is not None:
            instance.limit = limit
        if offset is not None:
            instance.offset = offset
        if region is not None:
            instance.region = region
        if sort_by is not None:
            instance.sort_by = sort_by
        if store_id is not None:
            instance.store_id = store_id
        if tags is not None:
            instance.tags = tags
        if target_namespace is not None:
            instance.target_namespace = target_namespace
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> QueryItems:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "activeOnly" in dict_ and dict_["activeOnly"] is not None:
            instance.active_only = bool(dict_["activeOnly"])
        elif include_empty:
            instance.active_only = False
        if "appType" in dict_ and dict_["appType"] is not None:
            instance.app_type = str(dict_["appType"])
        elif include_empty:
            instance.app_type = Union[str, AppTypeEnum]()
        if "availableDate" in dict_ and dict_["availableDate"] is not None:
            instance.available_date = str(dict_["availableDate"])
        elif include_empty:
            instance.available_date = ""
        if "baseAppId" in dict_ and dict_["baseAppId"] is not None:
            instance.base_app_id = str(dict_["baseAppId"])
        elif include_empty:
            instance.base_app_id = ""
        if "categoryPath" in dict_ and dict_["categoryPath"] is not None:
            instance.category_path = str(dict_["categoryPath"])
        elif include_empty:
            instance.category_path = ""
        if "features" in dict_ and dict_["features"] is not None:
            instance.features = str(dict_["features"])
        elif include_empty:
            instance.features = ""
        if (
            "includeSubCategoryItem" in dict_
            and dict_["includeSubCategoryItem"] is not None
        ):
            instance.include_sub_category_item = bool(dict_["includeSubCategoryItem"])
        elif include_empty:
            instance.include_sub_category_item = False
        if "itemType" in dict_ and dict_["itemType"] is not None:
            instance.item_type = str(dict_["itemType"])
        elif include_empty:
            instance.item_type = Union[str, ItemTypeEnum]()
        if "limit" in dict_ and dict_["limit"] is not None:
            instance.limit = int(dict_["limit"])
        elif include_empty:
            instance.limit = 0
        if "offset" in dict_ and dict_["offset"] is not None:
            instance.offset = int(dict_["offset"])
        elif include_empty:
            instance.offset = 0
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "sortBy" in dict_ and dict_["sortBy"] is not None:
            instance.sort_by = [str(i0) for i0 in dict_["sortBy"]]
        elif include_empty:
            instance.sort_by = []
        if "storeId" in dict_ and dict_["storeId"] is not None:
            instance.store_id = str(dict_["storeId"])
        elif include_empty:
            instance.store_id = ""
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = str(dict_["tags"])
        elif include_empty:
            instance.tags = ""
        if "targetNamespace" in dict_ and dict_["targetNamespace"] is not None:
            instance.target_namespace = str(dict_["targetNamespace"])
        elif include_empty:
            instance.target_namespace = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "activeOnly": "active_only",
            "appType": "app_type",
            "availableDate": "available_date",
            "baseAppId": "base_app_id",
            "categoryPath": "category_path",
            "features": "features",
            "includeSubCategoryItem": "include_sub_category_item",
            "itemType": "item_type",
            "limit": "limit",
            "offset": "offset",
            "region": "region",
            "sortBy": "sort_by",
            "storeId": "store_id",
            "tags": "tags",
            "targetNamespace": "target_namespace",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "activeOnly": False,
            "appType": False,
            "availableDate": False,
            "baseAppId": False,
            "categoryPath": False,
            "features": False,
            "includeSubCategoryItem": False,
            "itemType": False,
            "limit": False,
            "offset": False,
            "region": False,
            "sortBy": False,
            "storeId": False,
            "tags": False,
            "targetNamespace": False,
        }

    @staticmethod
    def get_collection_format_map() -> Dict[str, Union[None, str]]:
        return {
            "sortBy": "csv",  # in query
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "appType": ["DEMO", "DLC", "GAME", "SOFTWARE"],  # in query
            "itemType": [
                "APP",
                "BUNDLE",
                "CODE",
                "COINS",
                "EXTENSION",
                "INGAMEITEM",
                "LOOTBOX",
                "MEDIA",
                "OPTIONBOX",
                "SEASON",
                "SUBSCRIPTION",
            ],  # in query
            "sortBy": [
                "name",
                "name:asc",
                "name:desc",
                "createdAt",
                "createdAt:asc",
                "createdAt:desc",
                "updatedAt",
                "updatedAt:asc",
                "updatedAt:desc",
                "displayOrder",
                "displayOrder:asc",
                "displayOrder:desc",
            ],  # in query
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Achievement Service (2.16.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_icon import ModelsIcon


class ModelsAchievementRequest(Model):
    """Models achievement request (models.AchievementRequest)

    Properties:
        achievement_code: (achievementCode) REQUIRED str

        default_language: (defaultLanguage) REQUIRED str

        description: (description) REQUIRED Dict[str, str]

        goal_value: (goalValue) REQUIRED float

        hidden: (hidden) REQUIRED bool

        incremental: (incremental) REQUIRED bool

        locked_icons: (lockedIcons) REQUIRED List[ModelsIcon]

        name: (name) REQUIRED Dict[str, str]

        stat_code: (statCode) REQUIRED str

        tags: (tags) REQUIRED List[str]

        unlocked_icons: (unlockedIcons) REQUIRED List[ModelsIcon]

        global_: (global) OPTIONAL bool
    """

    # region fields

    achievement_code: str  # REQUIRED
    default_language: str  # REQUIRED
    description: Dict[str, str]  # REQUIRED
    goal_value: float  # REQUIRED
    hidden: bool  # REQUIRED
    incremental: bool  # REQUIRED
    locked_icons: List[ModelsIcon]  # REQUIRED
    name: Dict[str, str]  # REQUIRED
    stat_code: str  # REQUIRED
    tags: List[str]  # REQUIRED
    unlocked_icons: List[ModelsIcon]  # REQUIRED
    global_: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_achievement_code(self, value: str) -> ModelsAchievementRequest:
        self.achievement_code = value
        return self

    def with_default_language(self, value: str) -> ModelsAchievementRequest:
        self.default_language = value
        return self

    def with_description(self, value: Dict[str, str]) -> ModelsAchievementRequest:
        self.description = value
        return self

    def with_goal_value(self, value: float) -> ModelsAchievementRequest:
        self.goal_value = value
        return self

    def with_hidden(self, value: bool) -> ModelsAchievementRequest:
        self.hidden = value
        return self

    def with_incremental(self, value: bool) -> ModelsAchievementRequest:
        self.incremental = value
        return self

    def with_locked_icons(self, value: List[ModelsIcon]) -> ModelsAchievementRequest:
        self.locked_icons = value
        return self

    def with_name(self, value: Dict[str, str]) -> ModelsAchievementRequest:
        self.name = value
        return self

    def with_stat_code(self, value: str) -> ModelsAchievementRequest:
        self.stat_code = value
        return self

    def with_tags(self, value: List[str]) -> ModelsAchievementRequest:
        self.tags = value
        return self

    def with_unlocked_icons(self, value: List[ModelsIcon]) -> ModelsAchievementRequest:
        self.unlocked_icons = value
        return self

    def with_global(self, value: bool) -> ModelsAchievementRequest:
        self.global_ = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "achievement_code"):
            result["achievementCode"] = str(self.achievement_code)
        elif include_empty:
            result["achievementCode"] = ""
        if hasattr(self, "default_language"):
            result["defaultLanguage"] = str(self.default_language)
        elif include_empty:
            result["defaultLanguage"] = ""
        if hasattr(self, "description"):
            result["description"] = {
                str(k0): str(v0) for k0, v0 in self.description.items()
            }
        elif include_empty:
            result["description"] = {}
        if hasattr(self, "goal_value"):
            result["goalValue"] = float(self.goal_value)
        elif include_empty:
            result["goalValue"] = 0.0
        if hasattr(self, "hidden"):
            result["hidden"] = bool(self.hidden)
        elif include_empty:
            result["hidden"] = False
        if hasattr(self, "incremental"):
            result["incremental"] = bool(self.incremental)
        elif include_empty:
            result["incremental"] = False
        if hasattr(self, "locked_icons"):
            result["lockedIcons"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.locked_icons
            ]
        elif include_empty:
            result["lockedIcons"] = []
        if hasattr(self, "name"):
            result["name"] = {str(k0): str(v0) for k0, v0 in self.name.items()}
        elif include_empty:
            result["name"] = {}
        if hasattr(self, "stat_code"):
            result["statCode"] = str(self.stat_code)
        elif include_empty:
            result["statCode"] = ""
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        if hasattr(self, "unlocked_icons"):
            result["unlockedIcons"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.unlocked_icons
            ]
        elif include_empty:
            result["unlockedIcons"] = []
        if hasattr(self, "global_"):
            result["global"] = bool(self.global_)
        elif include_empty:
            result["global"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        achievement_code: str,
        default_language: str,
        description: Dict[str, str],
        goal_value: float,
        hidden: bool,
        incremental: bool,
        locked_icons: List[ModelsIcon],
        name: Dict[str, str],
        stat_code: str,
        tags: List[str],
        unlocked_icons: List[ModelsIcon],
        global_: Optional[bool] = None,
    ) -> ModelsAchievementRequest:
        instance = cls()
        instance.achievement_code = achievement_code
        instance.default_language = default_language
        instance.description = description
        instance.goal_value = goal_value
        instance.hidden = hidden
        instance.incremental = incremental
        instance.locked_icons = locked_icons
        instance.name = name
        instance.stat_code = stat_code
        instance.tags = tags
        instance.unlocked_icons = unlocked_icons
        if global_ is not None:
            instance.global_ = global_
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsAchievementRequest:
        instance = cls()
        if not dict_:
            return instance
        if "achievementCode" in dict_ and dict_["achievementCode"] is not None:
            instance.achievement_code = str(dict_["achievementCode"])
        elif include_empty:
            instance.achievement_code = ""
        if "defaultLanguage" in dict_ and dict_["defaultLanguage"] is not None:
            instance.default_language = str(dict_["defaultLanguage"])
        elif include_empty:
            instance.default_language = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = {
                str(k0): str(v0) for k0, v0 in dict_["description"].items()
            }
        elif include_empty:
            instance.description = {}
        if "goalValue" in dict_ and dict_["goalValue"] is not None:
            instance.goal_value = float(dict_["goalValue"])
        elif include_empty:
            instance.goal_value = 0.0
        if "hidden" in dict_ and dict_["hidden"] is not None:
            instance.hidden = bool(dict_["hidden"])
        elif include_empty:
            instance.hidden = False
        if "incremental" in dict_ and dict_["incremental"] is not None:
            instance.incremental = bool(dict_["incremental"])
        elif include_empty:
            instance.incremental = False
        if "lockedIcons" in dict_ and dict_["lockedIcons"] is not None:
            instance.locked_icons = [
                ModelsIcon.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["lockedIcons"]
            ]
        elif include_empty:
            instance.locked_icons = []
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = {str(k0): str(v0) for k0, v0 in dict_["name"].items()}
        elif include_empty:
            instance.name = {}
        if "statCode" in dict_ and dict_["statCode"] is not None:
            instance.stat_code = str(dict_["statCode"])
        elif include_empty:
            instance.stat_code = ""
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        if "unlockedIcons" in dict_ and dict_["unlockedIcons"] is not None:
            instance.unlocked_icons = [
                ModelsIcon.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["unlockedIcons"]
            ]
        elif include_empty:
            instance.unlocked_icons = []
        if "global" in dict_ and dict_["global"] is not None:
            instance.global_ = bool(dict_["global"])
        elif include_empty:
            instance.global_ = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsAchievementRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsAchievementRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsAchievementRequest,
        List[ModelsAchievementRequest],
        Dict[Any, ModelsAchievementRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "achievementCode": "achievement_code",
            "defaultLanguage": "default_language",
            "description": "description",
            "goalValue": "goal_value",
            "hidden": "hidden",
            "incremental": "incremental",
            "lockedIcons": "locked_icons",
            "name": "name",
            "statCode": "stat_code",
            "tags": "tags",
            "unlockedIcons": "unlocked_icons",
            "global": "global_",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "achievementCode": True,
            "defaultLanguage": True,
            "description": True,
            "goalValue": True,
            "hidden": True,
            "incremental": True,
            "lockedIcons": True,
            "name": True,
            "statCode": True,
            "tags": True,
            "unlockedIcons": True,
            "global": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Session Browser Service."""

__version__ = ""
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

# session
from .wrappers import add_player_to_session
from .wrappers import add_player_to_session_async
from .wrappers import admin_delete_session
from .wrappers import admin_delete_session_async
from .wrappers import admin_get_session
from .wrappers import admin_get_session_async
from .wrappers import admin_query_session
from .wrappers import admin_query_session_async
from .wrappers import admin_search_sessions_v2
from .wrappers import admin_search_sessions_v2_async
from .wrappers import create_session
from .wrappers import create_session_async
from .wrappers import delete_session
from .wrappers import delete_session_async
from .wrappers import delete_session_local_ds
from .wrappers import delete_session_local_ds_async
from .wrappers import get_active_custom_game_sessions
from .wrappers import get_active_custom_game_sessions_async
from .wrappers import get_active_matchmaking_game_sessions
from .wrappers import get_active_matchmaking_game_sessions_async
from .wrappers import get_recent_player
from .wrappers import get_recent_player_async
from .wrappers import get_session
from .wrappers import get_session_async
from .wrappers import get_session_by_user_i_ds
from .wrappers import get_session_by_user_i_ds_async
from .wrappers import get_session_history_detailed
from .wrappers import get_session_history_detailed_async
from .wrappers import get_total_active_session
from .wrappers import get_total_active_session_async
from .wrappers import join_session
from .wrappers import join_session_async
from .wrappers import remove_player_from_session
from .wrappers import remove_player_from_session_async
from .wrappers import update_session
from .wrappers import update_session_async
from .wrappers import update_settings
from .wrappers import update_settings_async
from .wrappers import user_query_session
from .wrappers import user_query_session_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsCountServerResponse
from ..models import ModelsCountSessionResponse
from ..models import ModelsDetailedCountServerResponse
from ..models import ModelsListServerResponse
from ..models import ModelsListSessionResponse
from ..models import ModelsServer
from ..models import ResponseError

from ..operations.admin import CountServer
from ..operations.admin import CountServerDetailed
from ..operations.admin import CountSession
from ..operations.admin import DeleteLocalServer
from ..operations.admin import DeleteServer
from ..operations.admin import DeleteSession
from ..operations.admin import GetServer
from ..operations.admin import ListLocalServer
from ..operations.admin import ListServer
from ..operations.admin import ListSession


@same_doc_as(CountServer)
def count_server(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Count all managed servers (CountServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint counts all of dedicated servers in a namespace managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/count

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsCountServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CountServer.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CountServer)
async def count_server_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Count all managed servers (CountServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint counts all of dedicated servers in a namespace managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/count

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsCountServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CountServer.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CountServerDetailed)
def count_server_detailed(
    region: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get detailed count of managed servers in a region (CountServerDetailed)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint counts all of dedicated servers in a region managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/count/detailed

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

    Responses:
        200: OK - ModelsDetailedCountServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CountServerDetailed.create(
        region=region,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CountServerDetailed)
async def count_server_detailed_async(
    region: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get detailed count of managed servers in a region (CountServerDetailed)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint counts all of dedicated servers in a region managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/count/detailed

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

    Responses:
        200: OK - ModelsDetailedCountServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CountServerDetailed.create(
        region=region,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CountSession)
def count_session(
    region: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Count all sessions (CountSession)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required scope: social

    This endpoint count all of sessions in a namespace managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/sessions/count

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

    Responses:
        200: OK - ModelsCountSessionResponse (sessions listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CountSession.create(
        region=region,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CountSession)
async def count_session_async(
    region: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Count all sessions (CountSession)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required scope: social

    This endpoint count all of sessions in a namespace managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/sessions/count

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

    Responses:
        200: OK - ModelsCountSessionResponse (sessions listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CountSession.create(
        region=region,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteLocalServer)
def delete_local_server(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a local server (DeleteLocalServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required scope: social

    This endpoint deletes a specified local dedicated server from DB.
    Note that DSM has no ability to shutdown local DS.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/local/{name}

        method: DELETE

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        name: (name) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (server deleted)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteLocalServer.create(
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteLocalServer)
async def delete_local_server_async(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a local server (DeleteLocalServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required scope: social

    This endpoint deletes a specified local dedicated server from DB.
    Note that DSM has no ability to shutdown local DS.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/local/{name}

        method: DELETE

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        name: (name) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (server deleted)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteLocalServer.create(
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteServer)
def delete_server(
    pod_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a server in a region (DeleteServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required scope: social

    This endpoint deletes a specified dedicated server from DB and terminates the DS pod.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/{podName}

        method: DELETE

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        pod_name: (podName) REQUIRED str in path

    Responses:
        204: No Content - (server deleted)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (server not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteServer.create(
        pod_name=pod_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteServer)
async def delete_server_async(
    pod_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a server in a region (DeleteServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required scope: social

    This endpoint deletes a specified dedicated server from DB and terminates the DS pod.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/{podName}

        method: DELETE

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        pod_name: (podName) REQUIRED str in path

    Responses:
        204: No Content - (server deleted)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (server not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteServer.create(
        pod_name=pod_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteSession)
def delete_session(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a session in a region (DeleteSession)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SESSION [DELETE]

    Required scope: social

    This endpoint deletes a specified session and its corresponding match result from DB.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SESSION [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/sessions/{sessionID}

        method: DELETE

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in path

    Responses:
        204: No Content - (session deleted)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSession.create(
        session_id=session_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteSession)
async def delete_session_async(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a session in a region (DeleteSession)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SESSION [DELETE]

    Required scope: social

    This endpoint deletes a specified session and its corresponding match result from DB.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SESSION [DELETE]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/sessions/{sessionID}

        method: DELETE

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in path

    Responses:
        204: No Content - (session deleted)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSession.create(
        session_id=session_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetServer)
def get_server(
    pod_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query a server in a region (GetServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint queries a specified dedicated server from DB.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/{podName}

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        pod_name: (podName) REQUIRED str in path

    Responses:
        200: OK - ModelsServer (server queried)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (server not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetServer.create(
        pod_name=pod_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetServer)
async def get_server_async(
    pod_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Query a server in a region (GetServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint queries a specified dedicated server from DB.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/{podName}

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        pod_name: (podName) REQUIRED str in path

    Responses:
        200: OK - ModelsServer (server queried)

        401: Unauthorized - ResponseError (Unauthorized)

        404: Not Found - ResponseError (server not found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetServer.create(
        pod_name=pod_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListLocalServer)
def list_local_server(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all managed local servers (ListLocalServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint lists all of local dedicated servers in a namespace managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/local

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsListServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListLocalServer.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListLocalServer)
async def list_local_server_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all managed local servers (ListLocalServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint lists all of local dedicated servers in a namespace managed by this service.

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers/local

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsListServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListLocalServer.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListServer)
def list_server(
    count: int,
    offset: int,
    region: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all managed servers in a region (ListServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint lists all of dedicated servers in a namespace managed by this service.

    Parameter Offset and Count is Required

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListServer.create(
        count=count,
        offset=offset,
        region=region,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListServer)
async def list_server_async(
    count: int,
    offset: int,
    region: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all managed servers in a region (ListServer)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required scope: social

    This endpoint lists all of dedicated servers in a namespace managed by this service.

    Parameter Offset and Count is Required

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SERVER [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/servers

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListServerResponse (servers listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListServer.create(
        count=count,
        offset=offset,
        region=region,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListSession)
def list_session(
    count: int,
    offset: int,
    region: Optional[str] = None,
    with_server: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all managed sessions in a region (ListSession)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required scope: social

    This endpoint lists all of sessions in a namespace managed by this service.

    Parameter Offset and Count is Required

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/sessions

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

        with_server: (withServer) OPTIONAL bool in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListSessionResponse (sessions listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListSession.create(
        count=count,
        offset=offset,
        region=region,
        with_server=with_server,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListSession)
async def list_session_async(
    count: int,
    offset: int,
    region: Optional[str] = None,
    with_server: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all managed sessions in a region (ListSession)

    Required permission: ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required scope: social

    This endpoint lists all of sessions in a namespace managed by this service.

    Parameter Offset and Count is Required

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:DSM:SESSION [READ]

    Required Scope(s):
        - social

    Properties:
        url: /dsmcontroller/admin/namespaces/{namespace}/sessions

        method: GET

        tags: ["Admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        region: (region) OPTIONAL str in query

        with_server: (withServer) OPTIONAL bool in query

        count: (count) REQUIRED int in query

        offset: (offset) REQUIRED int in query

    Responses:
        200: OK - ModelsListSessionResponse (sessions listed)

        401: Unauthorized - ResponseError (Unauthorized)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListSession.create(
        count=count,
        offset=offset,
        region=region,
        with_server=with_server,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )
